/**
  ******************************************************************************
  * C Library for FT800 EVE module 
  ******************************************************************************
  * @author  Akos Pasztor    (http://akospasztor.com)
  * @file    spi.c
  * @brief   SPI functions
  *          This file contains the GPIO initializations and functions
  *          for SPI peripherials.
  ******************************************************************************
  * Copyright (c) 2014 Akos Pasztor. All rights reserved.
  ******************************************************************************
**/

#include "LPC17xx.h"
#include "spi.h"

/*** SPI INIT **********************************************************************/
void SPI_init(void)
{	
    /* FT800 pin configuration*/
	
	LPC_SC->PCONP       |= (1 << 10);           	/* Enable power to SSPI0 block */

	LPC_GPIO0->FIODIR   |=  (1<<6);            		/* P0.07 is output             */
	LPC_GPIO0->FIOPIN   |=  (1<<6);            		/* set P0.06 high (SSEL inact.)*/
	LPC_PINCON->PINSEL0 &= ~(3<<12);            	/* P0.06 SSEL (used as GPIO)   */

	/* SCK, MISO, MOSI are SSP pins. */
	LPC_PINCON->PINSEL0 &= ~(3<<14);              	/* P0.7 cleared               */
	LPC_PINCON->PINSEL0 |=  (2<<14);              	/* P0.7 SCK0                  */
	LPC_PINCON->PINSEL0 &= ~((3<<16) | (3<<18));  	/* P0.8, P0.9 cleared        */
	LPC_PINCON->PINSEL0 |=  ((2<<16) | (2<<18));  	/* P0.8 MISO0, P0.9 MOSI0    */

	LPC_SC->PCLKSEL0 &= ~(3<<20);               	/* PCLKSP0 = CCLK/4 ( 25MHz)   */
	LPC_SC->PCLKSEL0 |=  (1<<20);               	/* PCLKSP0 = CCLK   (100MHz)   */

	LPC_SSP1->CPSR = 250;                       	/* 72MHz / 250 = 400kBit      */
													/* maximum of 18MHz is possible*/    
	LPC_SSP1->CR0  = 0x0007;                    	/* 8Bit, CPOL=0, CPHA=0        */
	LPC_SSP1->CR1  = 0x0002;                    	/* SSP1 enable, master         */
	
}

void SPI_speedup(void)
{
	LPC_SSP1->CPSR = 8;//Fsck=Fpclk/8=9Mhz
}

/*** SEND **************************************************************************/
char SPI_send(char data)
{
	LPC_SSP1->DR = data;
	while (LPC_SSP1->SR & 0x10);                 /* Wait for transfer to finish */
	return (LPC_SSP1->DR); 	
}

/*** REC ***************************************************************************/
char SPI_rec(char address)
{      
	SPI_send(address);
	return SPI_send(0);
}

/*** FT800 SPI select **************************************************************/
void FT_spi_select(void)
{
	LPC_GPIO0->FIOCLR |= (1<<6);
}

/*** FT800 SPI deselect ************************************************************/
void FT_spi_deselect(void)
{
	LPC_GPIO0->FIOSET |= (1<<6);
}
