/*****************************************************************************
* Catalyst LPC1768 Sample Codes
* FT800 LCD Driver Sample
* https://goo.gl/ZNDL4D
* Version : 1.0.0
* R&D : Digitronic.ir Team			06/28/2018
* www.ECA.ir
* www.Eshop.ECA.ir
* www.Forum.ECA.ir
*****************************************************************************/

#include "LPC17xx.h"
#include "stdio.h"
#include "ft800.h"
#include "uart.h"
#include "spi.h"

extern void Debug_Init (void);

volatile uint32_t msTicks;                            /* counts 1ms timeTicks */
/*----------------------------------------------------------------------------
  SysTick_Handler
 *----------------------------------------------------------------------------*/
void SysTick_Handler(void) {
  msTicks++;                        /* increment counter necessary in Delay() */
}

void delay_ms(int dly)
{
	int i=msTicks;
	while(msTicks-i>dly);
}


uint8_t initFT800(void)
{   
	uint8_t dev_id = 0;                  	// Variable for holding the read device id   
	delay_ms(50);                          // Delay 50 ms for stability

	//WAKE
	HOST_CMD_ACTIVE();
	delay_ms(500);

	//Ext Clock
	HOST_CMD_WRITE(CMD_CLKEXT);         // Send CLK_EXT Command (0x44)

	//PLL (48M) Clock
	HOST_CMD_WRITE(CMD_CLK48M);         // Send CLK_48M Command (0x62)

	//Read Dev ID
	dev_id = HOST_MEM_RD8(REG_ID);      // Read device id
	
	if(dev_id != 0x7C)                  // Device ID should always be 0x7C
	{   
		printf("Device ID=0x%x\n", dev_id);
		return 1;
	}

	HOST_MEM_WR8(REG_GPIO, 0x00);			// Set REG_GPIO to 0 to turn off the LCD DISP signal
	HOST_MEM_WR8(REG_PCLK, 0x00);      		// Pixel Clock Output disable

	HOST_MEM_WR16(REG_HCYCLE, 548);         // Set H_Cycle to 548
	HOST_MEM_WR16(REG_HOFFSET, 43);         // Set H_Offset to 43
	HOST_MEM_WR16(REG_HSYNC0, 0);           // Set H_SYNC_0 to 0
	HOST_MEM_WR16(REG_HSYNC1, 41);          // Set H_SYNC_1 to 41
	HOST_MEM_WR16(REG_VCYCLE, 292);         // Set V_Cycle to 292
	HOST_MEM_WR16(REG_VOFFSET, 12);         // Set V_OFFSET to 12
	HOST_MEM_WR16(REG_VSYNC0, 0);           // Set V_SYNC_0 to 0
	HOST_MEM_WR16(REG_VSYNC1, 10);          // Set V_SYNC_1 to 10
	HOST_MEM_WR8(REG_SWIZZLE, 0);           // Set SWIZZLE to 0
	HOST_MEM_WR8(REG_PCLK_POL, 1);          // Set PCLK_POL to 1
	HOST_MEM_WR8(REG_CSPREAD, 1);           // Set CSPREAD to 1
	HOST_MEM_WR16(REG_HSIZE, 480);          // Set H_SIZE to 480
	HOST_MEM_WR16(REG_VSIZE, 272);          // Set V_SIZE to 272

	/* configure touch & audio */
	HOST_MEM_WR8(REG_TOUCH_MODE, 0x03);     	//set touch on: continous
	HOST_MEM_WR8(REG_TOUCH_ADC_MODE, 0x01); 	//set touch mode: differential
	HOST_MEM_WR8(REG_TOUCH_OVERSAMPLE, 0x0F); 	//set touch oversampling to max
	HOST_MEM_WR16(REG_TOUCH_RZTHRESH, 5000);	//set touch resistance threshold
	HOST_MEM_WR8(REG_VOL_SOUND, 0xFF);      	//set the volume to maximum

	/* write first display list */
	HOST_MEM_WR32(RAM_DL+0, CLEAR_COLOR_RGB(0,0,0));  // Set Initial Color to BLACK
	HOST_MEM_WR32(RAM_DL+4, CLEAR(1,1,1));            // Clear to the Initial Color
	HOST_MEM_WR32(RAM_DL+8, DISPLAY());               // End Display List

	HOST_MEM_WR8(REG_DLSWAP, DLSWAP_FRAME);           // Make this display list active on the next frame

	HOST_MEM_WR8(REG_GPIO_DIR, 0x80);                 // Set Disp GPIO Direction 
	HOST_MEM_WR8(REG_GPIO, 0x80);                     // Enable Disp (if used)
	HOST_MEM_WR16(REG_PWM_HZ, 0x00FA);                // Backlight PWM frequency
	HOST_MEM_WR8(REG_PWM_DUTY, 0x80);                 // Backlight PWM duty    

	HOST_MEM_WR8(REG_PCLK, 0x05);                     // After this display is visible on the LCD

	return 0;
}

/* Clear Screen */
void clrscr(void)
{
	cmd(CMD_DLSTART);
	cmd(CLEAR_COLOR_RGB(0,0,0));
	cmd(CLEAR(1,1,1));
	cmd(DISPLAY());
	cmd(CMD_SWAP);
}

void clrscrcolor(uint8_t red, uint8_t green, uint8_t blue)
{
	cmd(CMD_DLSTART);
	cmd(CLEAR_COLOR_RGB(red, green, blue));
	cmd(CLEAR(1,1,1));
	cmd(DISPLAY());
	cmd(CMD_SWAP);
}

void touch_calibrate(void)
{    
    cmd(CMD_DLSTART);
    cmd(CLEAR_COLOR_RGB(0, 0, 0));
    cmd(CLEAR(1, 1, 1));
    cmd(CMD_CALIBRATE);
    cmd(0);
    /*
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_A, 0x00007DB5);
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_B, 0xFFFFFEF6);
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_C, 0xFFF61664);
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_D, 0x00000048);
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_E, 0xFFFFB387);
    HOST_MEM_WR32(REG_TOUCH_TRANSFORM_F, 0x0119F874);
	*/
}

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) 
{
	int i,led=0;

	SystemInit();
	LPC_GPIO0->FIODIR |= 1<< 20 ;
	
	Debug_Init();
	
	SPI_init();
	printf("Start main loop\r\n");
	while(initFT800()) 
	{
		printf("Try to Init FT800\r\n");
		delay_ms(500);
	}
	delay_ms(500);
	SPI_speedup();
	clrscr();
	
	while(1)
	{
		printf("main loop\r\n");
		cmd(CMD_DLSTART);
		cmd(CLEAR(1,1,1));
		cmd_gradient(0,0,0x000080, 0,250,0xA1E1FF);
		cmd_text(240, 90, 30, OPT_CENTER, "Please wait...");
		cmd_spinner(240, 150, 0, 0);
		cmd(DISPLAY());
		cmd(CMD_SWAP);
		delay_ms(5000);
		clrscrcolor(255, 0, 0);
		delay_ms(2000);
		clrscrcolor(0, 255, 0);
		delay_ms(2000);
		clrscrcolor(0, 0, 255);
		delay_ms(2000);
	}                        
}

